
#include <iostream> 
#include <windows.h>
#include "piex/piex.h"
#include "piex/piex_types.h" 
#include <fstream> 
#include <vector>
#if defined(_MSC_VER) || defined(__ANDROID_API__)
#define USE_OMP
#endif
#ifndef USE_OMP
#include <chrono>
auto const epoch = std::chrono::steady_clock::now();
double now() {
	return std::chrono::duration_cast<std::chrono::milliseconds>(std::chrono::steady_clock::now() - epoch).count() / 1000.0;
};
#else
#include <omp.h>
auto const epoch = omp_get_wtime();
double now() {
	return omp_get_wtime() - epoch;
};
#endif
template<typename FN>
double bench(const FN &fn) {
	auto took = -now();
	return (fn(), took + now());
}
#include <chrono>
#include <thread>
void sleep(double secs) {
	std::chrono::microseconds duration((int)(secs * 1000000));
	std::this_thread::sleep_for(duration);
}


class FileStream : public piex::StreamInterface
{
public:
	FileStream(const std::string & file)
	{
		if (file.empty()) {
			buffer.clear();
			bufferSize = 0;
			return;
		}
		std::ifstream ifs(file.c_str(), std::ios::binary);
		if (!ifs.good()) {
			buffer.clear();
			bufferSize = 0;
			return;
		}
		std::vector<unsigned char> curBuffer((std::istreambuf_iterator<char>(ifs)), (std::istreambuf_iterator<char>()));
		buffer.swap(curBuffer);
		bufferSize = buffer.size();

	}
	FileStream(std::vector<unsigned char>& abuffer)
	{
		int bufsize = abuffer.size();
		if (bufsize > 0)
		{
			buffer.resize(bufsize);
			memcpy(buffer.data(), abuffer.data(), bufsize*sizeof(unsigned char));
			bufferSize = bufsize;
		}
	}
	~FileStream(){ bufferSize = 0; buffer.clear(); };
	piex::Error GetData(const size_t offset, const size_t length,
		unsigned char* data)
	{
		if (bufferSize > 0)
		{
			unsigned char *readOffset = &buffer[offset];
			unsigned char *endBuffer = &buffer[bufferSize - 1];
			if ((readOffset + length) <= endBuffer)
			{
				memcpy(data, readOffset, length);
				return	piex::kOk;
			}
			else
			{
				return	piex::kFail;
			}
		}
		return	piex::kUnsupported;
	}

	std::vector<unsigned char>& GetBuffer()
	{
		return	buffer;
	}
protected:
	std::vector<unsigned char> buffer;
	int  bufferSize = 0;
};

bool writefile(const std::string &filename, const std::vector< char>& buffer) {
	if (!buffer.empty()) {
		std::ofstream ofs(filename.c_str(), std::ios::binary);
		ofs.write(&buffer[0], buffer.size());
		return ofs.good();
	}
	return false;
}
int main(int argc, char **argv) {

	std::cout << "Google Preview Image Extractor(PIEX) Demo" << std::endl;
	std::cout << "Ŀַ:https://github.com/google/piex" << std::endl;
	std::cout << "-----------------------------------------------------" << std::endl;
	std::cout << "ֽ֧ͼʽ: " << std::endl;
	std::cout << "ARW, CR2, DNG, NEF, NRW, " << std::endl;
	std::cout << "ORF, PEF, RAF, RW2, SRW" << std::endl;
	std::cout << " JPGͼʽ." << std::endl;
	std::cout << "Demo By Gaozhihan (Build 2016-01-19)" << std::endl;
	std::cout << "˲: http://tntmonks.cnblogs.com/" << std::endl;
	std::cout << "-----------------------------------------------------" << std::endl;
	if (argc < 2) {
		std::cout << "÷: " << argv[0] << " rawImage [rawImage [...]]" << std::endl;
		return -1;
	}
	std::string szfile;
	std::string savefile;

	for (int i = 1; i < argc; ++i) {
		szfile = argv[i];
		std::cout << "ļ: " << std::endl << szfile.c_str() << std::endl;
		char drive[_MAX_DRIVE];
		char dir[_MAX_DIR];
		char fname[_MAX_FNAME];
		char ext[_MAX_EXT];
		_splitpath_s(szfile.c_str(), drive, dir, fname, ext);
		savefile += drive;
		savefile += dir;
		savefile += fname;
		savefile += "_";
		std::vector< char> previewBuffer;
		std::vector< char> previewThumbnail;

		try {
			double costTime = bench([&]{
				FileStream fsfile(szfile.c_str());
				piex::StreamInterface* data_stream = &fsfile;
				piex::PreviewImageData rawPreviewImage;
				piex::Error piexErr = piex::GetPreviewImageData(data_stream, &rawPreviewImage);
				if (piexErr == piex::Error::kFail) {
					std::cout << "ʽļ." << std::endl;
				}
				else if (piexErr == piex::Error::kUnsupported) {
					std::cout << "ʽļݲ֧." << std::endl;
				}
				else
				{
					std::cout << " Чraw ʽ,ȡǶԤ." << std::endl;
					previewBuffer.resize(rawPreviewImage.preview_length);
					memcpy(previewBuffer.data(), fsfile.GetBuffer().data() + rawPreviewImage.preview_offset, rawPreviewImage.preview_length);

					previewThumbnail.resize(rawPreviewImage.thumbnail_length);
					memcpy(previewThumbnail.data(), fsfile.GetBuffer().data() + rawPreviewImage.thumbnail_offset, rawPreviewImage.thumbnail_length);
				}
			});
			std::cout << " ʱ: " << int(costTime * 1000) << " ms" << std::endl;

			std::string previewFile = savefile;
			previewFile += "_preview.jpg";

			std::string thumbnailFile = savefile;
			thumbnailFile += "_thumbnail.jpg";
			costTime = bench([&]{
				writefile(previewFile, previewBuffer);
				writefile(thumbnailFile, previewThumbnail);
			});
			std::cout << " ʱ: " << int(costTime * 1000) << " ms" << std::endl;
			ShellExecuteA(NULL, "open", previewFile.c_str(), NULL, NULL, SW_SHOW);
		}
		catch (...) {
			std::cout << "\r" << std::endl;
		}
	}
	std::cout << "ϣ˳." << std::endl;
	getchar();
	return 0;
}
